#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "program2.h"

#define D5_ON (PORTD |= (1<<PD5))
#define D5_OFF (PORTD &= ~(1<<PD5))
#define D5_TOGGLE (PORTD ^= (1<<PD5))

#define W_OFF PORTB = 0b00001111
#define W1_ON PORTB = 0b00000111
#define W2_ON PORTB = 0b00001011
#define W3_ON PORTB = 0b00001101
#define W4_ON PORTB = 0b00001110
#define PORT_W PORTC

#define S1 (!(PINA & (1<<PA0)))
#define S2 (!(PINA & (1<<PA1)))

unsigned char znaki[19] =
{
0b11000000,//0
0b11111001,//1
0b10100100,//2
0b10110000,//3
0b10011001,//4
0b10010010,//5
0b10000010,//6
0b11111000,//7
0b10000000,//8
0b10010000,//9
0b10001000,//a
0b10000011,//b
0b10100111,//c
0b10100001,//d
0b10000110,//e
0b10001110,//f
0b11111111,//puste
0b10101111,//r
0b10111111 //-
};

volatile int liczba = 0;
unsigned char w[4];

volatile unsigned int timer_ms = 0;
volatile int sekundy = 0;
volatile int minuty = 0;

volatile unsigned char nastawa_minuty = 5;
volatile unsigned char nastawa_sekundy = 0;

volatile unsigned char odliczanie = 0;
volatile unsigned int timer_klawisz = 0;

volatile unsigned char config_m = 0;
volatile unsigned char config_s = 0;

static inline void lcd_sendHalf(uint8_t data)
{
	if (data & (1<<0)) PORT(LCD_D4PORT) |= (1<<LCD_D4);
	else PORT(LCD_D4PORT) &= ~(1<<LCD_D4);

	if (data & (1<<1)) PORT(LCD_D5PORT) |= (1<<LCD_D5);
	else PORT(LCD_D5PORT) &= ~(1<<LCD_D5);

	if (data & (1<<2)) PORT(LCD_D6PORT) |= (1<<LCD_D6);
	else PORT(LCD_D6PORT) &= ~(1<<LCD_D6);

	if (data & (1<<3)) PORT(LCD_D7PORT) |= (1<<LCD_D7);
	else PORT(LCD_D7PORT) &= ~(1<<LCD_D7);
}

void _lcd_write_byte(unsigned char _data)
{
	SET_E;
	lcd_sendHalf(_data>>4);
	CLR_E;

	SET_E;
	lcd_sendHalf(_data);
	CLR_E;

	_delay_us(120);
}

void lcd_write_cmd(uint8_t cmd)
{
	CLR_RS;
	_lcd_write_byte(cmd);
}

void lcd_write_data(uint8_t data)
{
	SET_RS;
	_lcd_write_byte(data);
}

void lcd_cls(void)
{
	lcd_write_cmd( LCDC_CLS );
	_delay_ms(4.9);
}

void lcd_init()
{
	DDR(LCD_D7PORT) |= (1<<LCD_D7); //Data 7 for LCD
	DDR(LCD_D6PORT) |= (1<<LCD_D6); //Data 6 for LCD
	DDR(LCD_D5PORT) |= (1<<LCD_D5); //Data 5 for LCD
	DDR(LCD_D4PORT) |= (1<<LCD_D4); //Data 4 for LCD
	DDR(LCD_RSPORT) |= (1<<LCD_RS); //RS for LCD
	DDR(LCD_EPORT) |= (1<<LCD_E);   //E for LCD

	_delay_ms(15);
	PORT(LCD_EPORT) &= ~(1<<LCD_E);
	PORT(LCD_RSPORT) &= ~(1<<LCD_RS);
	SET_E;
	lcd_sendHalf(0x03);	//8-bit mode
	CLR_E;
	_delay_ms(4.1);

	SET_E;
	lcd_sendHalf(0x03);	//8-bit mode
	CLR_E;
	_delay_us(100);

	SET_E;
	lcd_sendHalf(0x03);	//8-bit mode
	CLR_E;
	_delay_us(100);

	SET_E;
	lcd_sendHalf(0x02);//4-bit mode
	CLR_E;
	_delay_us(100);

	//4-bit mode, 2 lines, 5x7 char
	lcd_write_cmd( LCDC_FUNC|LCDC_FUNC4B|LCDC_FUNC2L|LCDC_FUNC5x7 );
	//Cursor off
	lcd_write_cmd( LCDC_ONOFF|LCDC_CURSOROFF );
	//Display on
	lcd_write_cmd( LCDC_ONOFF|LCDC_DISPLAYON );
	//Moving cursor to the right without shifting the display
	lcd_write_cmd( LCDC_ENTRY|LCDC_ENTRYR );

	//Clear display
	lcd_cls();
}

void lcd_str(char* str)
{
	while (*str) lcd_write_data(*str++);
}

void lcd_locate(uint8_t y, uint8_t x)
{
	switch(y)
	{
		case 0: y = LCD_LINE1; break;
	    case 1: y = LCD_LINE2; break; //Address of 1st char in 2nd line
	}
	lcd_write_cmd( (0x80 + y + x) );
}

void lcd_defchar(uint8_t nr, uint8_t *def_znak)
{
	register uint8_t i,c;
	lcd_write_cmd( 64+((nr&0x07)*8) );
	for(i=0;i<8;i++)
	{
		c = *(def_znak++);
		lcd_write_data(c);
	}
}

int main()
{
	lcd_init(); //Initialisation for LCD
	
	//LCD
	lcd_locate(0, 0);
	//uint8_t buzka[] = {0,10,31,31,14,4,0, 0};

	//lcd_defchar(1, buzka);

	//lcd_str("\x01");
	lcd_str("ATmega32 program");
	//lcd_str("\x01");	
	lcd_locate(1, 0);
	lcd_str("   Gornik TBG   ");




	//DDRD |= (1<<PD5);
	OCR2 = 125;
	TCCR2 |= (1<<WGM21) | (1<<CS22) /*| (1<<CS21)*/ | (1<<CS20);
	TIMSK |= (1<<OCIE2);
	DDRC = 0xFF;
	DDRB = 0x0F;
	PORTA |= (1<<PA0) | (1<<PA1);
	//DDRD |= (1<<PD0);
	sei();
	minuty = nastawa_minuty;
	sekundy = nastawa_sekundy;

	unsigned char stan = 0;




	while(1)
	{
		switch (stan)
		{
			case 0:
			odliczanie = 0;
			minuty = nastawa_minuty;
			sekundy = nastawa_sekundy;
			stan = 1;
			break;

			case 1:
			if (S1) { PORTD |= (1<<PD0); timer_klawisz = 0; while(timer_klawisz < 100); stan = 2; }
			if (S2) { PORTD |= (1<<PD0); timer_klawisz = 0; while(timer_klawisz < 100); while(S2); stan = 3; }
			break;

			case 2:
			if (S1 && timer_klawisz >= 3000) { stan = 4; PORTD &= ~(1<<PD0); while(S1); }
			if (!S1 && timer_klawisz < 3000) stan = 0;
			break;

			case 3:
			PORTD &= ~(1<<PD0);
			odliczanie = 1;
			//timer_ms = 0;
			stan = 6;
			break;

			case 4:
			//config_m
			config_m = 1;
			if (S2) nastawa_minuty++;
			minuty = nastawa_minuty;
			if (nastawa_minuty == 60) nastawa_minuty = 0;
			timer_klawisz = 0;
			while(timer_klawisz < 200)
			{
				if (S1) config_m = 0, stan = 5;
			}
			break;

			case 5:
			//config_s
			config_s = 1;
			if (S2) nastawa_sekundy++;
			sekundy = nastawa_sekundy;
			if (nastawa_sekundy == 60) nastawa_sekundy = 0;
			timer_klawisz = 0;
			while(timer_klawisz < 200)
			{
				if (S1) config_s = 0, stan = 0;
			}
			break;

			case 6:
			if (S2) { while(S2); odliczanie = 0; stan = 0; }
			if (odliczanie == 1 && minuty == 0 && sekundy == 0) odliczanie = 0, PORTD |= (1<<PD0), stan = 7;
			break;

			case 7:
			if(S1) { while (S1); PORTD &= ~(1<<PD0); stan = 0; }
			break;

		}
	}
}

ISR(TIMER2_COMP_vect)
{
static unsigned char count = 0;

int l = liczba;
	if ((l >= -999) && (l <= 9999))
	{
		if (l < 0) l = l * -1;
		w[0] = (l / 1000);
		w[1] = (l / 100) % 10;
		w[2] = (l / 10) % 10;
		w[3] = (l / 1) % 10;

	  switch (count)
	  {
		case 0:
		{
			W_OFF;
			if (((liczba < 1000) && liczba >= 0) || ((liczba > -100) && (liczba <= -10)) || ((liczba > -10) && (liczba < 0))) PORT_W = znaki[16];
			else if (liczba <= -100) PORT_W = znaki[18];		
			else PORT_W = znaki[w[0]];
			if (config_m == 0 || timer_ms < 500) W1_ON;
			else W_OFF;
			break;
		}
		case 1:
		{
			W_OFF;
			/*if (((liczba < 100) && (liczba >= 0)) || ((liczba > -10) && (liczba <= 0))) PORT_W = znaki[16];
			else if ((liczba > -100) && (liczba <= -10)) PORT_W = znaki[18];
			else */PORT_W = znaki[w[1]];			
			PORT_W &= ~(1<<7); //kropka
			if (config_m == 0 || timer_ms < 500) W2_ON;
			else W_OFF;
			break;
		}
		case 2:
		{
			W_OFF;
			/*if ((liczba < 10) && (liczba >= 0)) PORT_W = znaki[16];
			else if ((liczba > -10) && (liczba < 0)) PORT_W = znaki[18];
			else */PORT_W = znaki[w[2]];
			if (config_s == 0 || timer_ms < 500) W3_ON;
			else W_OFF;
			break;
		}

		case 3:
		{
			W_OFF;
			PORT_W = znaki[w[3]];
			if (config_s == 0 || timer_ms < 500) W4_ON;
			else W_OFF;
			break;
		}
	  }
	}
	else
	{
	  switch (count)
	  {
		case 0:
		{
			W_OFF;
			PORT_W = znaki[14];
			W1_ON;
			break;
		}
		case 1:
		{
			W_OFF;
			PORT_W = znaki[17];
			W2_ON;
			break;
		}
		case 2:
		{
			W_OFF;
			PORT_W = znaki[17];
			W3_ON;
			break;
		}
		case 3:
		{
			W_OFF;
			PORT_W = znaki[16];
			W4_ON;
			break;
		}
      }
	}
		count++;
		if (count > 3) count = 0;

		timer_ms++;
		if (odliczanie)
		{
			if (timer_ms == 1000) timer_ms = 0, sekundy--;
			if (sekundy < 0) sekundy = 59, minuty--;
			if (minuty == 0 && sekundy == 0) odliczanie = 0;
		}
		else if (timer_ms == 1000) timer_ms = 0;
		timer_klawisz++;
		//liczba = timer_ms;
		liczba = (minuty * 100) + sekundy;
}
